/* build_index.cpp
 * Copyright (C) 2010, Francisco Claude, all rights reserved.
 *
 * Test program to build an int-based SSA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include <ssa_words.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <wt_coder.h>

using namespace std;

int main(int argc, char ** argv) {
  if(argc!=3) {
    cout << "usage: " << argv[0] << " <int-file> <index-file>" << endl;
    return 0;
  }

  fstream input(argv[1],ios::in | ios::binary);
  if(!input.is_open()) {
    cout << "Error opening file: " << argv[1] << endl;
    return -1;
  }

  input.seekg(0,ios::end);
  uint n=input.tellg()/4;
  uint * text = new uint[n+1];

  input.seekg(0,ios::beg);
  input.read((char*)text,sizeof(uint)*n);
  input.close();
  for(uint i=0;i<n;i++) text[i]++;
  text[n]=0;

  ssa_words * ssa = new ssa_words(text,n);
  static_bitsequence_builder * sbb = new static_bitsequence_builder_brw32(20);
  alphabet_mapper * am = new alphabet_mapper_none();
  wt_coder * wc = new wt_coder_huff(text,n+1,am);
  static_sequence_builder * ssb = new static_sequence_builder_wvtree(wc,sbb,am);
  ssa->set_static_sequence_builder(ssb);
  ssa->set_samplepos(8);
  ssa->set_samplesuff(8);
  ssa->build_index();

  cout << "Index size: " << ssa->size() << endl;

  FILE * fp = fopen(argv[2],"w");
  cout << "Save: " << ssa->save(fp) << endl;
  fclose(fp);

  delete sbb;
  delete ssa;
  delete [] text;

  return 0;
}
