/* test_locate.cpp
 * Copyright (C) 2010, Francisco Claude, all rights reserved.
 *
 * Program to test "locate" in the int-based SSA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#include <ssa_words.h>
#include <iostream>
#include <fstream>
#include <sstream>

using namespace std;

bool brute_check(uint * text, uint n, uint * pattern, uint m, uint * occ, uint ocs) {
  uint ret = 0;
  for(uint i=0;i<n-m;i++) {
    uint match_index=0;
    for(;match_index<m;match_index++)
      if(text[i+match_index]!=pattern[match_index])
        break;
    if(match_index==m) {
      //cout << "match index: " << i << endl;
      if(ret==ocs || occ[ret]!=i) {
        cout << "ret=" << ret << endl;
        cout << "ocs=" << ocs << endl;
        cout << "match position: " << i << endl;
        if(ret<ocs)
          cout << "next occ: " << occ[ret] << endl;
        //for(uint k=0;k<ocs;k++)
        //  cout << "my matches: " << occ[k] << endl;
        return false;
      }
      ret++;
    }
  }
  return true;
}


int main(int argc, char ** argv) {
  if(argc!=5) {
    cout << "usage: " << argv[0] << " <int-file> <index-file> <pattern-length> <repetitions>" << endl;
    return 0;
  }

  fstream input(argv[1],ios::in | ios::binary);
  if(!input.is_open()) {
    cout << "Error opening file: " << argv[1] << endl;
    return -1;
  }

  input.seekg(0,ios::end);
  uint n=input.tellg()/4;
  uint * text = new uint[n+1];

  input.seekg(0,ios::beg);
  input.read((char*)text,sizeof(uint)*n);
  input.close();
  //for(uint i=0;i<n;i++) text[i]++;
  text[n] = 0;

  bool err;
  FILE * fp = fopen(argv[2],"r");
  ssa_words * ssa = new ssa_words(fp,err);
  fclose(fp);
  ssa->print_stats();

  uint m;
  {
    stringstream ss;
    ss << argv[3];
    ss >> m;
  }

  uint rep;
  {
    stringstream ss;
    ss << argv[4];
    ss >> rep;
  }

  uint total_ocs = 0;
  uint * pattern = new uint[m+1];
  for(uint i=0;i<rep;i++) {
    uint pos = rand()%(n-m-1);
    for(uint k=0;k<m;k++)
      pattern[k] = text[pos+k];
    pattern[m] = 0;
    uint * occ;
    uint ocs = ssa->locate(pattern,m,&occ);
    sort(occ,occ+ocs);
    if(!brute_check(text,n,pattern,m,occ,ocs)) {
      cout << "Error for pattern " << i+1 << endl;
      delete [] occ;
      break;
    }
    total_ocs += ocs;
    delete [] occ;
  }

  cout << "Total ocs: " << total_ocs << endl;

  delete [] pattern;
  delete ssa;
  delete [] text;

  return 0;
}
